/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2014  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 4.02a                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit_PIC24F for Microchip PIC24F CPU

Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS
          Feel free to modify this file acc. to your target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

//
// Pragmas for configuration bits
// Auto-detected by MPLAB X and merged into compiled code
//
//
#define FNOSC_PRIPLL  0xFBFF
#define POSCMOD_XT    0xFFFD
#define _CONFIG2(x) __attribute__((section("__CONFIG2.sec,code"))) int _CONFIG2 = (x);
_CONFIG2(FNOSC_PRIPLL & POSCMOD_XT) // Primary XT OSC with PLL x4
#define FWDTEN        0xFF7F
#define _CONFIG1(x) __attribute__((section("__CONFIG1.sec,code"))) int _CONFIG1 = (x);
_CONFIG1(FWDTEN) // Watchdog disabled

/*********************************************************************
*
*       Clock frequency settings
*/
#ifndef   OS_FSYS                   /* CPU Main clock frequency     */
  #define OS_FSYS 32000000uL        /* Assume 32MHz                 */
#endif


#ifndef   OS_PCLK_TIMER             /* Peripheral clock for timer   */ 
  #define OS_PCLK_TIMER (OS_FSYS/2) /* May vary from CPU clock      */
#endif                              /* depending on CPU             */

#ifndef   OS_PCLK_UART              /* Peripheral clock for UART    */
  #define OS_PCLK_UART (OS_FSYS/2)  /* May vary from CPU clock      */
#endif                              /* depending on CPU             */

#ifndef   OS_TICK_FREQ
  #define OS_TICK_FREQ  (1000)
#endif

/*********************************************************************
*
*       Configuration of communication to OSView
*/
#ifndef   OS_VIEW_ENABLE            // Global enable of communication
  #define OS_VIEW_ENABLE    (1)     // Default: on
#endif

#ifndef   OS_VIEW_USE_UART          // If set, UART will be used for communication
  #define OS_VIEW_USE_UART  (1)     // Default: 1 => Uart is used
#endif                              // if OS_VIEW_ENABLE is on

/*********************************************************************
*
*       UART settings for embOSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to embOSView, please define it to be -1
*       Currently UART2 is supported and the standard
*       setup enables UART2 per default
*/
#ifndef   OS_UART
  #define OS_UART (2)
#endif    

#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE (38400)
#endif

/****** End of configurable options *********************************/

#define OS_UART_USED  ((OS_VIEW_ENABLE && (OS_VIEW_USE_UART != 0)) && (OS_UART == 2))

#if OS_UART_USED
  #define OS_COM_INIT() OS_COM_Init()
#else
  #define OS_COM_INIT()
#endif

#define OS_TIMER_RELOAD (OS_PCLK_TIMER / OS_TICK_FREQ)

/*********************************************************************
*
*       Check configuration
*
**********************************************************************
*/

#ifndef   DEBUG     /* Should normally be defined as project option */
  #define DEBUG  (0)
#endif

/*********************************************************************
*
*       Local defines (sfrs used in RTOSInit.c)
*
**********************************************************************
*/
#define IFS0       (*(volatile OS_U16*) 0x0084)
#define IFS1       (*(volatile OS_U16*) 0x0086)
#define IEC0       (*(volatile OS_U16*) 0x0094)
#define IEC1       (*(volatile OS_U16*) 0x0096)
#define IPC0       (*(volatile OS_U16*) 0x00A4)
#define IPC7       (*(volatile OS_U16*) 0x00B2)
#define TMR1       (*(volatile OS_U16*) 0x0100)
#define PR1        (*(volatile OS_U16*) 0x0102)
#define T1CON      (*(volatile OS_U16*) 0x0104)
#define U2MODE     (*(volatile OS_U16*) 0x0230)
#define U2STA      (*(volatile OS_U16*) 0x0232)
#define U2TXREG    (*(volatile OS_U16*) 0x0234)
#define U2RXREG    (*(volatile OS_U16*) 0x0236)
#define U2BRG      (*(volatile OS_U16*) 0x0238)
#define TRISF      (*(volatile OS_U16*) 0x02DE)
#define OSCCONH    (*(volatile OS_U16*) 0x0742)
#define OSCCONL    (*(volatile OS_U16*) 0x0743)
#define RCON       (*(volatile OS_U16*) 0x0740)
#define OSCCON     (*(volatile OS_U16*) 0x0742)
#define CLKDIV     (*(volatile OS_U16*) 0x0744)
#define PLLFBD     (*(volatile OS_U16*) 0x0746)
#define OSCTUN     (*(volatile OS_U16*) 0x0748)

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       _T1InterruptHandler()
*       Systick interrupt handler function
*/
static void _T1InterruptHandler(void) {
  IFS0 &= ~(1 << 3);                   // Reset Timer 1 interrupt pending flag   
  OS_TICK_Handle();                    // Call OS tick handler                 
}

/*********************************************************************
*
*       _T1Interrupt()
*       The interruput service routine
*/
void OS_ISR _T1Interrupt(void) {
  OS_CallNestableISR(&_T1InterruptHandler);
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*       Initialize the hardware (timer) required for the OS to run.
*       May be modified, if an other timer should be used
*/
void OS_InitHW(void) {
  OS_IncDI();
  //
  // Use timer1 as embOS system timer
  //
  PR1    = OS_TIMER_RELOAD;            // Reload value
  T1CON  = 0x8000;                     // Timer control register, prescale 1/1
  IFS0  &= ~(1 << 3);                  // Reset Timer 1 interrupt pending flag   
  IEC0  |= (1 << 3);                   // Enable timer1 ints
  IPC0   = 0x1000;                     // Set timer interrupt priority to 1
  //
  // Initialize the optional UART for OS viewer
  //
  OS_COM_INIT();
  OS_DecRI();
} 

/*********************************************************************
*
*       OS_Idle()
*
*       Please note:
*       This is basically the "core" of the idle loop.
*       This core loop can be changed, but:
*       The idle loop does not have a stack of its own, therefore no
*       functionality should be implemented that relies on the stack
*       to be preserved. However, a simple program loop can be programmed
*       (like toggeling an output or incrementing a counter)
*/
void OS_Idle(void) {    // Idle loop: No task is ready to execute
  while (1) {           // Nothing to do ... wait for interrupt
    #if (DEBUG == 0)
                        // Switch CPU into sleep mode
    #endif
  }
}

/*********************************************************************
*
*       OS_GetTime_Cycles()
*
*       This routine is required for task-info via OSView or high
*       resolution time measurement functions.
*       It returns the system time in timer clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  OS_U32 Time;
  OS_U32 Cnt;

  Time = OS_GetTime32();
  Cnt  = TMR1;
  //
  // Check if timer interrupt pending ...
  //
  if (IFS0 & (1u << 3u)) { // Check timer interrupt pending flag 
    Cnt = TMR1;            // Interrupt pending, re-read timer and adjust result
    Time++;     
  }
  return (OS_TIMER_RELOAD * Time) + Cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us()
*
*       Convert Cycles into micro seconds.
*
*       If your clock frequency is not a multiple of 1 MHz,
*       you may have to modify this routine in order to get proper
*       diagnostics.
*
*       This routine is required for profiling or high resolution time
*       measurement only. It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_PCLK_TIMER/1000000);
}

/*********************************************************************
*
*       Optional communication with embOSView
*
**********************************************************************
*/
#if OS_UART_USED
/*
* Uart register value and baudrate mistake calculation
*/
#define BAUDRATEREG (OS_PCLK_UART/16/OS_BAUDRATE-1)
#if BAUDRATEREG > 255
  #error Cannot set up UART2 for the OS_FSYS and OS_BAUDRATE. Please correct values. 
#endif

/*********************************************************************
*
*       _U2RXInterruptHandler() 
*       USART Rx handler
*/
static void _U2RXInterruptHandler(void) {
  int c;
  IFS1 &= ~(1 << 14);                  // Reset uart2 rx interrupt pending flag
  c = U2RXREG;
  if ((U2STA & 0x0E) == 0x00) {        // Check if errors occured
    OS_OnRx(c);
  }
}

/*********************************************************************
*
*       _U2RXInterrupt() 
*       USART Rx interrupt service routine
*/
void OS_ISR _U2RXInterrupt(void) {
  OS_CallNestableISR(&_U2RXInterruptHandler);
}

/*********************************************************************
*
*       _U2TXInterruptHandler()
*       USART Tx handler
*/
static void _U2TXInterruptHandler(void) {
  IFS1 &= ~(1 << 15);       // Reset uart2 tx interrupt pending flag
  if (OS_OnTx()) {          // No more characters to send ? 
    IEC1 &= ~(1ul << 15);   // Disable Tx int
  }
}

/*********************************************************************
*
*       _U2TXInterrupt() OS USART interrupt handler
*       USART Tx interrupt service routine
*/
void OS_ISR _U2TXInterrupt(void) {
  OS_CallNestableISR(&_U2TXInterruptHandler);
}

/*********************************************************************
*
*       OS_COM_Send1()
*       Send one character via UART.
*       Never call this function from your application
*/
void OS_COM_Send1(OS_U8 c) {
  IEC1   |= (1ul << 15);   // Enable Tx int
  U2TXREG = (OS_U16) c;    // Store character in Tx register
}

/*********************************************************************
*
*       OS_COM_Init()
*       Initialize UART for OSView
*/
void OS_COM_Init(void) {        
  TRISF   = 0x0010;                       // Set port pins
  U2BRG   = BAUDRATEREG;                  // Set uart baudrate
  U2MODE  = (1ul << 15);                  // Set low baud enable bit and enable uart  
  U2STA   = 0x0440;                       // Select int cause and enable transmit 
  IFS1   &= ~((1ul << 15) | (1ul << 14)); // Reset uart2 tx and rx interrupt pending flag
  IPC7    = 0x1100;                       // Set priority to 1
  IEC1   |= (1ul << 14);                  // Enable Rx int
}

#else  /* UART for communication not used, define dummy functions */

void OS_COM_Send1(OS_U8 c) {
  OS_USEPARA(c);           /* Avoid compiler warning */
  OS_COM_ClearTxActive();  /* Let the OS know that Tx is not busy */
}

#endif /*  OS_UART_USED  */


/****** End Of File *************************************************/
